// This class contains a sorted queue of events that will be occurring, which makes the class name
// a bit of a misnomer.  There is a single EventList in the Simulation class.  Whenever a new event
// is scheduled to happen, the server where it will happen calls addEvent on this class, and the
// EventList will place that event in the queue according to the time when the event will happen.

public class EventList extends Object
{
  private EventQueue eventQueue; // an event queue extends our queue class, and has the pushSort method

  /*
    pre: none
    post: constructs an eventList, makes an empty EventQueue
  */
  public EventList()
  { this.eventQueue = new EventQueue();
  }

  /*
    pre: where is not null
    post: creates a new event at Server 'where' and time 'time', which will be located at a point in the queue
          such that the event ahead of it has a time earlier, and the event behind has a time later
  */
  public void addEvent(int time,Server where)
  { //d.w("adding event at time " + time + " at location " + where.getName());
    eventQueue.pushSort(new Event(time,where));

  }
  /*
    pre:  the EventQueue actually has stuff in it
    post: returns the time of the event at the top of the queue
  */
  public int getNextEventTime()
  { Event useEvent = (Event)eventQueue.peek(); // grabs the top event.  If the program is running properly, this will never be null
    return useEvent.getTime(); // returns the time of the next event
  }

  /*
    pre: none
    post: if there is still events left in the queue that occur at time 'time', then it returns the top one.
          if there are no more events left in the queue the occur at time 'time' , then it returns null
          since the queue is sorted, if the top event does not have time 'time', then there are clearly no more events at time 'time'.
  */
  public Event getNextEvent(int time)
  { Event tempEvent = (Event)eventQueue.peek(); // gets the top event
    if(tempEvent != null) // if the top event actually exists
    {
      if(tempEvent.getTime() == time) // if the top event has the same time as was requested, return it
      { return (Event)(eventQueue.pop());
      }
    }
    return null; // if there are no events or there are none with time 'time', then return null
  }
}