// This class represents the entrance.  It contains all the data, seeds, and ranges it needs
// to create people according to the specs we are given

public class PeopleMaker extends Server
{ private double meanInter; // the mean inter-arrival time
  private Random mySecRandom; // random dealing with security
  private Random myCusRandom; // random dealing with customs
  private Random myTimeRandom; // random dealing with inter-arrival time
  private Random myExitsRandom; // where people go
  private double[] probs; // probabilities of people going to the various desks
  private double cPercent; // the % of people that are going to customs
  private double s1Percent; // the % of people going to security 1
  private double s2Percent; // the % of people going to security 2
  private int numPeople; // number of people generated

  public PeopleMaker(double[] probs,Server[] linkedTo,String myName, EventList master,
                     long seederExit,double meanInter,long seedSec,
                     long seedCustom,double cPercent,long seedA,double s1Percent,double s2Percent)
  { super(linkedTo,myName,master,0,0); // creates a server
    this.probs = probs;
    this.meanInter = meanInter;
    this.mySecRandom = new Random(seedSec); // security
    this.myCusRandom = new Random(seedCustom); // customs
    this.myTimeRandom = new Random(seedA); // inter-arrival time seed
    this.myExitsRandom = new Random(seederExit);
    this.eventMaster = master;
    this.cPercent = cPercent;
    this.s1Percent = s1Percent;
    this.s2Percent = s2Percent;
    this.numPeople = 0;
    this.linkedTo = linkedTo;
    //d.w("Constructin' shit");
  }
  /*
    pre: nothing
    post: Since this will only be called when the EventList thinks an event is occuring here, this will
          create a new person, with all that person's destination properties set.  It will then add them
          to a check-in desk's queue.
  */
  public void Run(int currentTime)
  {

    // adds an event occuring a random amount of time from the current time.  The event occurs at this server when the next person shows up

    // figures out if this person is going to customs
    boolean goingToCustoms = (myCusRandom.nextUniformInt(1,100) < (this.cPercent * 100));
    // figures out the time of arrival of the next person
    double timeOfArrival = currentTime + (int)(myTimeRandom.nextExp(this.meanInter));
    // if this random number is bigger than the percentage of people going to security 1, then this person is going to security 2
    boolean whichSec = (mySecRandom.nextUniformInt(1,100) > (this.s1Percent * 100));

    // adds a new event at the next time of arrival determined
    this.eventMaster.addEvent((int)(timeOfArrival),this);

    // temp for holding which security this person goes to
    int iWhichSec;
    if(whichSec) // if their random # was > the % of people going to sec 1, then they are going to sec 2
    { iWhichSec = 1;
    }
    else
    { iWhichSec=0;
    }
    // constructs the person
    Person temp = new Person(Constants.SEC_NAMES[iWhichSec],goingToCustoms,currentTime);

    // sends the person off to some link
    this.sendToLink(temp,currentTime);

    // keeps count of how many people we've created
    this.numPeople++;
    this.stats.setPeople(numPeople);

    //d.w("CurrentTime = " + currentTime);
  }

  /*
    pre: who is not null
    post: sends a person off to the check-in counter that they are destined for
  */
  public void sendToLink(Person who,int currentTime)
  { double randNum = ((double)this.myExitsRandom.nextUniformInt(1,100))/100;

    double total = 0;
    double lastTotal = 0;
    for(int x=0;x<this.probs.length;x++) // goes through all the probabilities
    { total += this.probs[x]; // keeps an upper-bound of the 'window' of probabilities we are looking at
      if(randNum >= lastTotal && randNum <= total) // if the random number went through the current 'window', then they are going to the exit corresponding to that window
      {
        //d.w("Sending person to " + this.linkedTo[x].getName() + " at time " + currentTime);
        this.linkedTo[x].addPerson(who,currentTime);
      }
      lastTotal = total;
    }
  }

  /*
    pre: none
    post: returns if we've finished off all the people yet
  */
  public boolean Done()
  { return (this.numPeople >= Constants.MAX_PEOPLE);
  }
}